#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""
This package provides a way to distribute useful database management tools as
part of the Python package of the placement service: openstack-placement.

Their filenames may be discovered using ``pkg_resources`` to look in the
``placement_db_tools`` package:

    pkg_resources.resource_filename('placement_db_tools',
                                    'mysql-migrate-db.sh')
"""
